# Copyright (c) 2000-2010 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

"""Service for handling Komodo views (basically tabs in the main window(s))."""

from xpcom import components, nsError, ServerException, COMException
from xpcom.client import WeakReference
from xpcom.server import WrapObject, UnwrapObject

import logging
log = logging.getLogger('koViewService')

# XXX the view service needs some additional thought for supporting multi
# window capability.

class koViewService:
    _com_interfaces_ = [components.interfaces.koIViewService,
                        components.interfaces.nsIWindowMediatorListener]
    _reg_desc_ = "Komodo View Service Component"
    _reg_contractid_ = "@activestate.com/koViewService;1"
    _reg_clsid_ = "{7F78C1E7-A746-449F-951D-8BED1B502CCD}"
    
    def __init__(self):
        self._viewMgr = {}
        self.wrapped = WrapObject(self, components.interfaces.nsIWindowMediatorListener)

        self.wm = components.classes["@mozilla.org/appshell/window-mediator;1"].\
                        getService(components.interfaces.nsIWindowMediator);
        self.wm.addListener(self.wrapped)

    def onWindowTitleChange(self, xulwindow, newTitle):
        pass
    def onOpenWindow(self, xulwindow):
        pass
    def onCloseWindow(self, xulwindow):
        # we're given a nsIXULWindow, not a nsIDOMWindow; so we need to check
        # everything we know to see if it belongs to that nsIXULWindow and
        # remove everything that matches (usually just one).
        ci = components.interfaces
        for window in self._viewMgr.keys():
            xw = window.QueryInterface(ci.nsIInterfaceRequestor).\
                        getInterface(ci.nsIWebNavigation).\
                        QueryInterface(ci.nsIDocShellTreeItem).\
                        treeOwner.\
                        QueryInterface(ci.nsIInterfaceRequestor).\
                        getInterface(ci.nsIXULWindow)
            if xw == xulwindow:
                del self._viewMgr[window]

    def setViewMgr(self, viewMgr):
        window = self.wm.getMostRecentWindow('Komodo')
        self._viewMgr[window] = viewMgr

    def get_currentView(self):
        window = self.wm.getMostRecentWindow('Komodo')
        if window in self._viewMgr:
            return self._viewMgr[window].currentView
        else:
            log.error("Trying to get currentView from the koViewService but no viewMgr has been set")

    def get_topView(self):
        window = self.wm.getMostRecentWindow('Komodo')
        if window in self._viewMgr:
            return self._viewMgr[window].topView
        else:
            log.error("Trying to get topView from the koViewService but no viewMgr has been set")
