/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: JavaScript; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Komodo's Sleuth main component
 *
 * Manages Test Plan Containers
 */

if (typeof(ko)=='undefined') {
    var ko = {};
}
if (!('sleuth' in ko)) {
    ko.sleuth = {}; // A namespace
}
(function() {

var sleuthLog = ko.logging.getLogger('sleuth.chrome');
// sleuthLog.setLevel(ko.logging.LOG_DEBUG);

var Sleuth = function() {
    this.gSleuthSvc = Components.classes["@activestate.com/koSleuthService;1"].
                  getService(Components.interfaces.koISleuthService);
    this.TESTPLAN_PREFNAME = this.gSleuthSvc.TESTPLAN_PREFNAME;
}; 
ko.sleuth.manager = new Sleuth(); // The sleuth singleton

Sleuth.prototype.createNewTestPlan = function sl_createNewTestPlan(projectURL) {
    if (projectURL) {
        this._createNewTestPlan_withContainer(ko.projects.manager.getProjectByURL(projectURL),
                                              'project');
    } else {
        this._createNewTestPlan_withContainer(null, 'global');
    }
};

Sleuth.prototype.createNewDocumentTestPlan = function sl_createNewDocumentTestPlan(documentURL) {
    this._createNewTestPlan_withContainer(ko.views.manager.getViewForURI(documentURL).koDoc, 'document');
};

Sleuth.prototype._createNewTestPlan_withContainer = function sl__createNewTestPlan_withContainer(prefContainer, prefContainerType) {
    ko.windowManager.openOrFocusDialog(
        "chrome://komodo/content/sleuth/createNewTestPlan.xul",
        'komodo_tpcp',
        "chrome,modal,resizable,close=yes",
        ko, prefContainer, prefContainerType);
}

Sleuth.prototype._getGlobalTestPlanNames = function _getGlobalTestPlanNames(sorted) {
    if (typeof(sorted) == "undefined") {
        sorted = false;
    }
    var prefset = this._getGlobalPrefs();
    if (!prefset.hasPref(this.TESTPLAN_PREFNAME)) {
        return [];
    }
    return this.getTestPlanNamesFromPrefs(prefset.getPref(this.TESTPLAN_PREFNAME),
                                           sorted);
};

Sleuth.prototype._getDocumentTestPlanNames = function _getDocumentTestPlanNames(sorted) {
    if (typeof(sorted) == "undefined") {
        sorted = false;
    }
    var prefset = this._getGlobalPrefs();
    if (!prefset.hasPref(this.TESTPLAN_PREFNAME)) {
        return [];
    }
    return this.getTestPlanNamesFromPrefs(prefset.getPref(this.TESTPLAN_PREFNAME),
                                           sorted);
};

// This works for document test plans as well -- first arg is a prefs container,
// second arg is the name of the attribute giving the prefset

Sleuth.prototype._getProjectTestPlanNames = function _getProjectTestPlanNames(prefContainer, prefAttr, sorted) {
    if (typeof(sorted) == "undefined") {
        sorted = false;
    }
    var prefset = prefContainer[prefAttr];
    if (!prefset.hasPrefHere(this.TESTPLAN_PREFNAME)) {
        return [];
    }
    return this.getTestPlanNamesFromPrefs(prefset.getPref(this.TESTPLAN_PREFNAME),
                                           sorted);
};

Sleuth.prototype.getTestPlanNamesFromPrefs = function getTestPlanNamesFromPrefs(sleuthPrefs, sorted) {
    var testplan_names = {};
    sleuthPrefs.getPrefIds(testplan_names, {});
    var names = testplan_names.value;
    if (sorted) {
        names.sort(function(a, b) {
                a = a.toLowerCase();
                b = b.toLowerCase();
                return (a < b ? -1 :
                        a > b ? 1 : 0);
            });
    }
    return names;
};

Sleuth.prototype._getGlobalPrefs = function _getGlobalPrefs() {
    if (!this._globalPrefSvc) {
        this._globalPrefSvc = Components.classes["@activestate.com/koPrefService;1"].getService(Components.interfaces.koIPrefService);
    }
    return this._globalPrefSvc.prefs;
};

Sleuth.prototype.getNumProjectTestPlans = function getNumProjectTestPlans(project, prefattr) {
    return this._getProjectTestPlanNames(project, prefattr).length;
}

Sleuth.prototype.getNumGlobalTestPlans = function getNumGlobalTestPlans() {
    return this._getGlobalTestPlanNames().length;
}

Sleuth.prototype._compareStringsUI = function(a, b) {
    var alc = a.toLowerCase();
    var blc = b.toLowerCase();
    return ((alc < blc)
            ? -1 :
            (alc > blc) ? 1 : 0);
}

Sleuth.prototype.initHarnessPopupMenu = function initHarnessPopupMenu(menulist, harness_indices) {
    var popupmenu = menulist.childNodes[0];
    while (popupmenu.hasChildNodes()) {
        popupmenu.removeChild(popupmenu.lastChild);
    }
    var o = {};
    this.gSleuthSvc.get_harness_names(o, {});
    var harness_names = o.value;
    harness_names.sort(this._compareStringsUI);
    for (var i = 0;  i < harness_names.length; i++) {
        var menuitem = document.createElement("menuitem");
        menuitem.setAttribute("label",  harness_names[i]);
        harness_indices[harness_names[i]] = i;
        popupmenu.appendChild(menuitem);
    }
};

// This function uses the results of initHarnessPopupMenu to find
// the version 4.3 info for each harness.
// initHarnessPopupMenu [out]: { lang : { harnessName:name, menuIdx:num }}

Sleuth.prototype.getLangInfoFromHarnessIndices = function(harness_indices,
                                                          harnessFromLanginfo) {
    for (var harness_name in harness_indices) {
        var lang = this.gSleuthSvc.language_from_harness(harness_name);
        if (!lang) {
            continue;
        }
        if (!(lang in harnessFromLanginfo)
            || this.gSleuthSvc.is_v43_compatible(harness_name)) {
            harnessFromLanginfo[lang] = {
                harnessName:harness_name,
                menuIdx:    harness_indices[harness_name]
            }
        }
    }
};

Sleuth.prototype.updateMainDocumentTestPlanMenu = function sl_updateMainDocumentTestPlanMenu(popupmenu) {
    var addConditionalRunMenu = true;
    return this.updateDocumentTestPlanMenu(popupmenu, addConditionalRunMenu);
};

Sleuth.prototype.updateProjectTestPlanMenu = function sl_updateProjectTestPlanMenu(popupmenu) {
    var project = ko.projects.manager.getSelectedProject();
    if (!project) {
        while (popupmenu.hasChildNodes()) {
            var lastChild = popupmenu.lastChild;
            if (lastChild.hasAttribute("keep")) {
                break;
            }
            popupmenu.removeChild(lastChild);
        }
        menuitem = document.createElement("menuitem");
        menuitem.setAttribute("label", "No Project Available");
        menuitem.setAttribute("disabled", true);
        menupopup.appendChild(menuitem);
        return;
    }
    var testPlans = this._getProjectTestPlanNames(project, 'prefset', true);
    this._fillPopupMenu(popupmenu, testPlans,
                        "doSelectedProjectTestPlan", 
                        "createNewTestPlan",
                        project.url,
                        false);
}

Sleuth.prototype.updateDocumentTestPlanMenu = function sl_updateDocumentTestPlanMenu(popupmenu, addConditionalRunMenu) {
    if (typeof(addConditionalRunMenu) == "undefined") {
        addConditionalRunMenu = false;
    }
    var currDocument = ko.views.manager.currentView.koDoc;
    var testPlans = (currDocument
                     ? this._getProjectTestPlanNames(currDocument, 'prefs', true)
                     : []);
    this._fillPopupMenu(popupmenu, testPlans,
                        "doSelectedDocumentTestPlan",
                        "createNewDocumentTestPlan",
                        currDocument.file.URI, addConditionalRunMenu);
}

Sleuth.prototype._fillPopupMenu = function sl_updateProjectTestPlanMenu(popupmenu, testPlans, methodName, createMethodName, containerURI, addConditionalRunMenu) {
    // First clear the popupmenu
    while (popupmenu.hasChildNodes()) {
        var lastChild = popupmenu.lastChild;
        if (lastChild.hasAttribute("keep")) {
            break;
        }
        popupmenu.removeChild(lastChild);
    }
    var menuitem;
    var label;

    //XXX Move the handler into the command-observer...
    var needSeparator = false;
    if (addConditionalRunMenu && ko.sleuth.tabManager.isReady()) {
        needSeparator = true;
        menuitem = document.createElement("menuitem");
        label = "Run Current Test Plan"; //XXX I18N this
        // We don't get command key shortcuts this way, right?
        menuitem.setAttribute("label", label);
        menuitem.setAttribute("accesskey", "R");
        menuitem.setAttribute("oncommand",
                              "ko.sleuth.tabManager.rerunTests()");
        popupmenu.appendChild(menuitem);
    }

    if (testPlans.length > 0) {
        if (needSeparator) {
            popupmenu.appendChild(document.createElement("menuseparator"));
        } else {
            needSeparator = true;
        }
        for (var i = 0; i < testPlans.length; i++) {
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", testPlans[i]);
            menuitem.setAttribute("accesskey",
                                  testPlans[i].substr(0, 1).toUpperCase());
            var handler = ("ko.sleuth.manager." + methodName + "('"
                           + containerURI
                           + "', "
                           + i
                           + ");");
            menuitem.setAttribute("oncommand", handler);
            popupmenu.appendChild(menuitem);
        }
    }

    if (needSeparator) {
        popupmenu.appendChild(document.createElement("menuseparator"));
    }
    menuitem = document.createElement("menuitem");
    label = "Create New Test Plan..."; //XXX I18N this
    menuitem.setAttribute("label", label);
    menuitem.setAttribute("accesskey", "C");
    menuitem.setAttribute("oncommand",
                          ("ko.sleuth.manager." + createMethodName + "('"
                           + containerURI + "')"));
    popupmenu.appendChild(menuitem);

    if (methodName == "doSelectedDocumentTestPlan"
        && this.getNumGlobalTestPlans() > 0) {
        menuitem = document.createElement("menuitem");
        label = "Move Global Test Plans to Document..."; //XXX I18N this
        menuitem.setAttribute("label", label);
        menuitem.setAttribute("accesskey", "M");
        menuitem.setAttribute("oncommand",
                            ("ko.sleuth.manager.launchImportGlobalTestPlans('"
                               + containerURI + "')"));
        popupmenu.appendChild(menuitem);
    }

};

Sleuth.prototype.addProjectTestPlanNamesToPopup = function(event) {
    return this.updateProjectTestPlanMenu(event.target);
};

Sleuth.prototype.doSelectedProjectTestPlan = function(prjURL, testPlanIndex) {
    var projectManager = ko.projects.manager;
    var project = projectManager.getProjectByURL(prjURL);
    if (projectManager.getCurrentProject() != project) {
        projectManager.setCurrentProject(project);
    }
    var testPlans = this._getProjectTestPlanNames(project, 'prefset', true);
    var sleuthPrefs = project.prefset.getPref(this.TESTPLAN_PREFNAME);
    ko.sleuth.tabManager.setProject(project, testPlans[testPlanIndex]);
    this._doSelectedTestPlan(sleuthPrefs, testPlans, testPlanIndex);
};

Sleuth.prototype.doSelectedDocumentTestPlan = function(docURI, testPlanIndex) {
    var selectedView = ko.views.manager.getViewForURI(docURI);
    var selectedDoc =  selectedView.koDoc;
    if (ko.views.manager.currentView.koDoc != selectedDoc) {
        ko.views.manager.currentView = selectedDoc;
    }
    var testPlans = this._getProjectTestPlanNames(selectedDoc, 'prefs', true);
    var sleuthPrefs = selectedDoc.prefs.getPref(this.TESTPLAN_PREFNAME);
    ko.sleuth.tabManager.setDocument(selectedDoc);
    this._doSelectedTestPlan(sleuthPrefs, testPlans, testPlanIndex);
};

Sleuth.prototype.doSelectedGlobalTestPlan = function(urlNotUsed, testPlanIndex) {
    var testPlans = this._getGlobalTestPlanNames(true);
    var sleuthPrefs = this._getGlobalPrefs().getPref(this.TESTPLAN_PREFNAME);
    ko.sleuth.tabManager.clearProject();
    ko.sleuth.tabManager.setProject(null, testPlans[testPlanIndex]);
    this._doSelectedTestPlan(sleuthPrefs, testPlans, testPlanIndex);
};

Sleuth.prototype._doSelectedTestPlan = function _doSelectedTestPlan(sleuthPrefs, testPlans, testPlanIndex) {
    var testPlanName = testPlans[testPlanIndex];
    if (!testPlanName) {
        dump("Can't find a test plan for test " + testPlanName + "\n");
        return;
    }
    var testPlanPref = sleuthPrefs.getPref(testPlanName);
    ko.sleuth.tabManager.show();
    var absPath = this.dirFromPrefs(testPlanPref);
    ko.sleuth.tabManager.runTests(testPlanName,
                                  absPath,
                                  testPlanPref.getStringPref("language"),
                                  testPlanPref.getStringPref("command_line"));
};

Sleuth.prototype.launchImportGlobalTestPlans = function(docURI) {
    ko.windowManager.openOrFocusDialog(
        "chrome://komodo/content/sleuth/importGlobalTestPlans.xul",
        'komodo_import_global_testplan',
        "chrome,modal,resizable,close=yes",
        ko, docURI);
};

Sleuth.prototype._getFirstRecognizedLanguage_CheckPart = function _getFirstRecognizedLanguage_CheckPart(part, candidates, currDepth) {
    var childrenObj = {};
    var countObj = {};
    part.getChildren(childrenObj, countObj);
    var children = childrenObj.value;
    for (var i = 0; i < children.length; i++) {
        var child = children[i];
        switch(child.type) {
        case "livefolder":
            candidates.push([currDepth, child]);
            break;
        case "file":
            var language = this.langRegistrySvc.suggestLanguageForFile(child.getFile().baseName);
            if (language in this.supported_languages) {
                return language;
            }
        }
    }
    return null;
};

Sleuth.prototype.getFirstRecognizedProjectLanguage = function sl_getFirstRecognizedProjectLanguage(project, supported_languages, maxDepth) {
    // Do a breadth-first-search walk of the project,
    // looking for filetypes we recognize
    // Set maxDepth to 0 (or a negative value) to never stop prematurely.
    // e.g.: Setting maxDepth to 1 looks only at the files at the first level
    // of the project.
    
    if (typeof(maxDepth) == "undefined") {
        maxDepth = 3;
    } else if (maxDepth == 0) {
        maxDepth = -1;
    }
    this.supported_languages = supported_languages;
    this.langRegistrySvc = (Components.classes['@activestate.com/koLanguageRegistryService;1'].
                           getService(Components.interfaces.koILanguageRegistryService));
    var candidates = [[maxDepth, project]];
    // Do a breadth-first-search on the project.
    var data;
    while (candidates.length > 0) {
        data = candidates.shift();
        if (data[0] == 0) {
            break;
        }
        var lang = this._getFirstRecognizedLanguage_CheckPart(data[1], candidates, data[0] - 1);
        if (lang) {
            return lang;
        }
    }
    return null;
    
};

// Pref handlers:
// project-pref directories will be stored relative to the project's path
// in the project's pref, if we write them in as URIs.

Sleuth.prototype.dirFromPrefs = function dirFromPrefs(prefset) {
    var directoryURI = prefset.getStringPref("directory");
    // convert from URI to filepath
    if (!directoryURI) {
        dump("Sleuth.prototype.dirFromPrefs: No directory pref in current set\n");
        return ko.window.getCwd();
    }
    var directoryPath = ko.uriparse.URIToLocalPath(directoryURI);
    //dump("read dir pref URI: "
    //     + directoryURI
    //     + ",\n local: "
    //     + directoryPath
    //     + "\n");
    return directoryPath;
};

Sleuth.prototype.setPrefToDir = function setPrefToRelativeDir(prefset, directoryPath) {
    var directoryURI = ko.uriparse.pathToURI(directoryPath);
    if (directoryURI && directoryURI[directoryURI.length - 1] != '/') {
        directoryURI += '/';
    }
    //dump("writing dir pref: "
    //     + directoryPath
    //     + ",\n uri: "
    //     + directoryURI
    //     + "\n");
    prefset.setStringPref("directory", directoryURI);
};

//XXX: Disable this command when !ko.sleuth.tabManager.isReady()
Sleuth.prototype.handleRunCurrentTestPlan = function handleRunCurrentTestPlan()
{
    if (ko.sleuth.tabManager.isReady()) {
        ko.sleuth.tabManager.rerunTests();
    }
};

})();

