ko.collaboration.socket = {};

(function() {
    var obsSvc = Components.classes['@mozilla.org/observer-service;1']
                           .getService(Components.interfaces.nsIObserverService);
    // Reference to `this` for callback functions.
    var this_ = this;
    var host = ko.collaboration.service.prefs.getCharPref("socketio.host");
    var port = ko.collaboration.service.prefs.getCharPref("socketio.port");
    var secure = ko.collaboration.service.prefs.getBoolPref("socketio.useTLS");
    var socket = new io.Socket(host, {
        'port': port,
        'secure': secure,
        // There might be an issue with multipart XHR in
        // mozilla: http://groups.google.com/group/socket_io/browse_thread/thread/6d3725378ca8b12c
        // I think I saw it, too, but cannot reproduce
        // it. If it's an issue, just switch to polling.
        transports: ['xhr-multipart'],
        rememberTransport: false,
        // Even if we have only one transport enabled,
        // set this to true so connect_failed is fired.
        tryTransportsOnConnectTimeout:  true,
        connectTimeout: 5000
    });
    
    
    // Socket handler functions:
    
    socket.on('connect', function() {
        window.clearTimeout(this_._autoConnectTimeout);
        this_._autoConnectTimeout = null;
        this_.authenticate();
    })
    
    socket.on('message', function(message) {
        if (message == 'friends') {
            ko.collaboration.friends.update();
        } else if (message == 'sessions') {
            ko.collaboration.sessions.refresh();
        } else if (message == 'mobwrite') {
            // Server has updates- sync asap.
            mobwrite.syncNow();
        } else if (message == 'auth:failed') {
            // FIXME Tell SSO that our credentials are invalid.
            obsSvc.notifyObservers(this_, 'services:sso:check-session', null);
            this_.disconnect();
        } else if (message == 'auth:ok') {
            this_.authenticated = true;
            obsSvc.notifyObservers(this_, 'collaboration:socket-connected', null);
        } else {
            // TODO
        }
    });
    
    socket.on('disconnect', function() {
        // FIXME I think we should just try to recover and not notify this. Only
        // notify when there is an unrecoverable error, such as failed
        // authentication or failed connect.
        obsSvc.notifyObservers(this_, 'collaboration:socket-disconnected', null);
        // If we have previously been authenticated and are still logged in,
        // the disconnect is likely a network issue. Try to reconnect.
        if (this_.authenticated && ko.collaboration.service.loggedIn)
            this_._delayedAutoConnect(100);
        this_.authenticated = false;
    });
    
    socket.on('connect_failed', function() {
        this_.authenticated = false;
        if (!this_._autoConnectTimeout)
            // Only notify on the first failed connection attempt.
            obsSvc.notifyObservers(this_, 'collaboration:socket-connect-failed', null);
        this_._delayedAutoConnect(5000);
    });
    
    this._autoConnectTimeout = null;
    this._delayedAutoConnect = function(timeout) {
        window.clearTimeout(this._autoConnectTimeout);
        this._autoConnectTimeout = window.setTimeout(function() {
            this_.connect();
        }, timeout);
    };
    
    this._socket = socket;
    
    this._events = ["services:sso:login-successful",
                    "services:sso:logout-successful",
                    "collaboration:force-reconnect"];
    
    this._addObserver = function() {
        for each (let event in this._events)
            obsSvc.addObserver(this, event, false); 
    };
    
    this._removeObserver = function() {
        for each (let event in this._events)
            obsSvc.removeObserver(this, event); 
    };
    
    this._unloadSocket = function() {
        this._removeObserver();
        // We don't want to notify a lost connection to the service when the
        // window is unloaded.
        this._socket.on('disconnect', function() {});
        this._socket.on('connect_failed', function() {});
        this._socket.disconnect();
        // FIXME socketio still throws an exception in `Transport._setTimeout`
        // during unload. Can't figure out why.
    };
    
    // Public methods:
    
    this.init = function() {
        this._addObserver();
        let this_ = this;
        window.addEventListener('beforeunload', function() {
            this_._unloadSocket();
        }, false);
        
        if (ko.collaboration.service.loggedIn)
            this.connect();
    };
    
    this.authenticated = false;
    
    this.observe = function(subject, topic, data) {
        switch(topic) {
            case "services:sso:login-successful":
                this.connect();
                break;
            case "services:sso:logout-successful":
                this.disconnect();
                break;
            case "collaboration:force-reconnect":
                this.connect();
                break;
        }
    };
    
    this.connect = function() {
        if (ko.collaboration.service.loggedIn)
            socket.connect();
    };
    
    this.disconnect = function() {
        socket.disconnect();
    };
    
    this.authenticate = function authenticate() {
        let authStr = ko.collaboration.service.ssoKey + "\n"
                      + mobwrite.syncUsername;
        socket.send(authStr);
    };
    
    this.__defineGetter__('connected', function() {
        return socket.connected;
    });
}).apply(ko.collaboration.socket);
ko.collaboration.socket.init();